#!/bin/sh

## RASTOW - rastow.sh
## Classify raster charts (*.kap extension) by latitude/longitude.
## Run on any Linux distro with awk.

## COPYRIGHT 2010-2016 Free Software Foundation, Inc. License GNU.
## GPL version 3 or later [ http://gnu.org/licenses/gpl.html ].
## This  is  free  software:  you  are free to use, change and redistribute it.
## There is NO WARRANTY, to the extent permitted by law.
## Navigation beta software, use it as your owownn risks !
## Frank Baudet - <http://marinux.tuxfamily.org>.

## HISTORY
## 2013/09 first public release > V0.1
## 2014/03 correct bug (with some distros) in "find" command parameters
##         better scan for dates editions
##         better internationalization in titles
##         add update function
##         add rename function
##         add datums in chartlist catalog > V0.2
## 2015/12 add help params > V0.2b
## 2016/01 add bsbfix function
##         remove -V option in sort for update operation > V0.3
## TODO: nothing is perfect...


help="\r\n
\r RASTOW - Classify raster charts (*.kap extension) by latitude/longitude.
\r
\r Usage: rastow.sh COMMAND SOURCEDIR [DESTDIR]
\r
\r COMMAND:
\r  copy       copy all files from SOURCEDIR to DESTDIR.
\r  link       create symlinks from SOURCEDIR to DESTDIR.
\r  update     set the timestamp of files from SOURCEDIR at edition creation.
\r  rename     rename all files from SOURCEDIR with the header's title.
\r  bsbfix     rewrite the index table from SOURCEDIR, editing it in place.
\r             bsbfix from libbsb package must be present on your system.
\r  chartlist  create only a chartlist from SOURCEDIR into this directory.
\r  help       or --help or -h: this help.
\r
\r SOURCEDIR:  the directory path which contains your charts. This directory can
\r             contain subdirectories.
\r DESTDIR:    the directory path for destination. If it not exist,
\r             it will be created.
\r
\r
\r *** RASTOW *** (raster-stow contraction) classify your raster charts
\r  collection (*.kap files) creating 96 subdirectories divided by timezone
\r (15° steps, without any country exception), north and south, above and
\r below 45° of latitude.
\r Two modes of creation are available: -copy- or symbolic -link-.
\r At the end of operation, a chartlist.txt containing a catalog of your charts
\r is automatically created. Using -chartlist- command, you can create only
\r a chartlist from your database.
\r In copy mode, the created database may be bigger than the original base
\r because a map can have a part in more than one timezone or <45° to >45°
\r in latitude, and it resides in all the timezone subs covered. So, use
\r symlinks if your space storage is critical.
\r Two others commands are useful to organize your collection with datas
\r founded in the file header:
\r <update> change the file date to the last update edition.
\r <rename> change the filename by the chart title.
\r It's a good practice to make an update before a rename or copy command:
\r if two files are equals, only the most recent will be handle.
\r Some files have trouble with non-ASCII caracters (ex: swedish).
\r Use the program detox to correct it. Probably some others manually.
\r
\r No guarantee - use this script under your own responsability.
\r GPL - General Public Lisense V3 - <http://gnu.org>.
\r Free to use, modify and distribute it.
\r Comments, suggests, english corrections of this text are welcome.
\r <http://marinux.tuxfamily.org>.
\r\n "


# don't harm
IN_DIR=
OUT_DIR=
METHOD=
CNT_IN=0


# cmdline filters
#################

case $1 in
   "help" | "--help" | "-h" | "")
      echo
      printf "$help"
      exit
   ;;
   "copy")
      echo
      echo " *** build database by copy ***"
      METHOD="copy"
   ;;
   "link")
      echo
      echo " *** build database by symbolics links ***"
      METHOD="sym_link"
   ;;
   "update")
      echo
      echo " *** update all charts by edition creation ***"
      METHOD="up_date"
   ;;
  "rename")
      echo
      echo " *** rename all charts by title ***"
      METHOD="re_name"
   ;;
   "bsbfix")
      echo
      echo " *** rewrite the index table of charts ***"
      METHOD="bsb_fix"
   ;;
   "chartlist")
      echo
      echo " *** create a database chartlist ***"
      METHOD="chart_list"
   ;;
   *)
      echo
      echo " *** please, specify a valid command ***"
      exit
   ;;
esac

case $2 in
   "")
      echo
      echo " *** please, specify a source directory ***"
      exit
   ;;
   *)
      ls "$2" >/dev/null
      if [ $? -ne 0 ]; then
         echo
         echo " *** your source path is not valid ***"
         exit
      else
         IN_DIR="$2"
      fi
   ;;
esac

case $3 in
   "")
      if [ "$METHOD" != "up_date" -a "$METHOD" != "re_name" -a "$METHOD" != "bsb_fix" -a "$METHOD" != "chart_list" ]; then
         echo
         echo " *** please, give an output path ***"
         exit
      fi
   ;;
   *)
      if [ "$METHOD" = "up_date" -o "$METHOD" = "re_name" -o "$METHOD" = "chart_list" ]; then
         echo
         echo " *** please, give only an input path with update, rename or chartlist command ***"
         exit
      else
         OUT_DIR="$3"
      fi
   ;;
esac



###################
# update function #
###################

up_date() {

   DIR="$1"
   # cleanup old tmp update.tmp
   [ -f /tmp/update.tmp ] && rm /tmp/update.tmp
   [ -f /tmp/update.err ] && rm /tmp/update.err

   # print header
   echo >/tmp/update
   echo "   ####################################################" >>/tmp/update
   echo "   ### batch update generated by rastow.sh - GPL 3+ ###" >>/tmp/update
   echo "   ####################################################" >>/tmp/rename
   echo "             # http://marinux.tuxfamily.org #" >>/tmp/update
   echo >>/tmp/update
   echo "   # see in /tmp/update.err if any error" >>/tmp/update
   echo >>/tmp/update


   # scan for kap files
   ####################
   find -L "$DIR" -iname "*.kap" | while read DIRFILE; do
      # limit lines header search
      # more of 640 in 1644DH01.kap (Brazil) founded
      #echo $DIRFILE
      FILENAME=$(basename "$DIRFILE")
      #echo $FILENAME
      head -n 680 ${DIRFILE} |
      # extract datas
      gawk -F "," -v in_dir=${DIR} -v dirfile=${DIRFILE} -v file_name=${FILENAME} \
      \
      '$3~/ED=/ {
         # update
         if (nd == "found") exit
         else update_full($3)
         # update

      }

      $2~/ND=/ {
         # other update prefix (Brasilian charts)
         nd = found
         update_full($2)
      }

      # date format
      function update_full(date) {

         # standard date format: yyyy-mm-dd
         if ((substr(date,8,1) == "-") && (substr(date,11,1) == "-")) {
            yyyy = substr(date, 4, 4)
            mm = substr(date, 9, 2)
            dd = substr(date, 12, 2)
            upd = "accurate"
         }

         # other format: dd/mm/yyyy or d/mm/yy or dd/m/yy etc...
         else {
            # look for day separator
            dd_sep = index(date,"/")
            # extract day
            dd = substr(date,4,(dd_sep - 4))
            # 2 digits
            if (length(dd) == 1) dd = "0" dd
            if (length(dd) == 0) dd = "01"
            #print dd

            # look for month separator
            mm_dd = substr(date,(dd_sep + 1))
            # extract month
            mm_sep = index(mm_dd,"/")
            mm = substr(mm_dd,1,(mm_sep - 1))
            # 2 digits
            if (length(mm) == 1) mm = "0" mm
            if (length(mm) == 0) mm = "01"
            #print mm

            # case of mm/dd/ (founded !)
            if (mm > 12) {
               mm_1 = mm
               dd_1 = dd
               dd = mm_1
               mm = dd_1
            }
            # some erratics dates
            if (mm == "00") mm = "01"
            if (dd == "00") dd = "01"

            # extract year
            yy = substr(mm_dd,(mm_sep + 1))
            # convert to numeric
            yy = yy + 0
            lg_yy = length(yy)

            # format year on 4 digits, year 2000
            if (lg_yy == 1) yyyy = 2000
            # only 2 digits given, arbitrary base on 1950
            else if (lg_yy == 2) {
               if (yy > 70) yyyy = "19" yy
               else yyyyyy = "20" yy
            }
            else if (lg_yy == 4) yyyy = yy
            #print yyyy
            upd = "accurate"

         }

         if (upd == "accurate") {
            print file_name " \x3C\x3C\x3C " yyyy " " mm " " dd
            print "touch -t " yyyy mm dd "0000 " "\x27" dirfile "\x27" >>"/tmp/update.tmp"
            #print "touch -t " "1971" "01" "01" "0000 " "\x27" dirfile "\x27" >>"/tmp/update.tmp"
         }
         else {
         # no edition date founded, or non conventional format
         print "### update impossible on " dirfile " see /tmp/update.err ###"
         print "no edition date founded, or non conventional format in " dirfile >>"/tmp/update.err"
         }
      }'

   done


   # classify and write final commands
   ###################################
   # don't harm
   LC_ALL=C

   sort -b /tmp/update.tmp >>/tmp/update

   # advise user
   #echo
   #[ -f /tmp/update ] && echo " *** batch file created ***"
   echo
   echo " *** update... ***"
   echo
   . /tmp/update && echo " *** maps updated by edition creation ***"
   echo

   exit
}

#######################
# end update function #

#  update execution   #
#######################

[ "${METHOD}" = "up_date" ] && up_date ${IN_DIR}

########################
# end update execution #
########################



######################
# rename function #
######################

re_name() {

   DIR="$1"
   # cleanup old tmp rename.tmp
   [ -f /tmp/rename.tmp ] && rm /tmp/rename.tmp

   # print header
   echo >/tmp/rename
   echo "   ####################################################" >>/tmp/rename
   echo "   ### batch rename generated by rastow.sh - GPL 3+ ###" >>/tmp/rename
   echo "   ####################################################" >>/tmp/rename
   echo "             # http://marinux.tuxfamily.org #" >>/tmp/rename
   echo >>/tmp/rename


   # scan for kap files
   ####################
   find -L "$DIR" -iname "*.kap" | while read DIRFILE; do
      # limit lines header search
      # more of 640 in 1644DH01.kap (Brazil) founded
      #echo $DIRFILE
      FILENAME=$(basename "$DIRFILE")
      #echo $FILENAME
      head -n 680 ${DIRFILE} | iconv -f ISO_8859-1 |
      # extract datas
      gawk -F "," -v in_dir=${DIR} -v dirfile=${DIRFILE} -v file_name=${FILENAME} \
      \
      '$1~/BSB\/NA=/ {
         # chart title
         #print $1

         # only with readable title
         valid_char = gsub(/[[:alnum:]]/,"&",$1)
         if ( valid_char > 0 ) {
            # reformat and shortcut title
            #############################
            # upper/lower case very irregular, so capitalize all tiles
            lg_title = length($1)
            title = toupper(substr($1, 8, (lg_title - 7)))

            # remove mysterious apparition of CR
            gsub("\x0D$","",title)

            # remove spaces at the end of line
            space_end = ""
            while ( space_end != 0 ) space_end = gsub(" $","",title)

            # remove multiples spaces
            space_mult = gsub("  "," ",title)
            while ( space_mult >= 1 ) space_mult = gsub("  "," ",title)

            # replace """ by "`" (accute accent)
            gsub (/\x27/,"`",title)

            # replace <simple quote> by "`" (accute accent)
            #gsub (/\/,"`",title)

            # replace "." by "-"
            gsub (/\./,"-",title)

            # remove spaces before and after "-"
            gsub(" -","-",title)
            gsub("- ","-",title)

            # remove spaces after "(" and before ")"
            gsub(/\(\x20/,"(",title)
            gsub(/\x20\)/,")",title)

            # replace forbidden "/", "\", and ":" in filenames
            gsub("/","~",title)
            gsub(/\\/,"~",title)
            gsub(":","|",title)

            # remove spaces before and after characters replacement
            gsub(" ~","~",title)
            gsub("~ ","~",title)

            # replace spaces by underscore
            gsub(" ","_",title)
         }

            # advise user
            print title
            # format path file
            lg_file = length(file_name)
            lg_dirfile = length(dirfile)
            path_file = substr(dirfile, 1, (lg_dirfile - lg_file))
            #print path_file
            # write batch source
            print "mv -u " "\x27" dirfile "\x27" " " "\x27" path_file title ".KAP" "\x27" >>"/tmp/rename.tmp"

         }'

   done


   # classify and write final commands
   ###################################
   # don't harm
   LC_ALL=C

   sort -b -V /tmp/rename.tmp >>/tmp/rename

   # advise user
   #echo
   #[ -f /tmp/rename ] && echo "   ### batch file created ###"
   echo
   echo " *** renaming... ***"
   echo
   . /tmp/rename && echo " *** maps renamed by title ***"
   echo

   exit
}

#######################
# end rename function #

#  rename execution   #
#######################

[ "${METHOD}" = "re_name" ] && re_name ${IN_DIR}

########################
# end rename execution #
########################



###################
# bsbfix function #
###################

bsb_fix() {

   if ! (which bsbfix) >/dev/null 2>&1; then
      echo
      echo " *** no bsbfix found, operation canceled ***"
      echo
      exit
   else
      # advise user
      echo
      echo " *** WORKING HARD *** "
      echo
   fi

   DIR="$1"
   # scan for kap files
   ####################
   find -L "$DIR" -iname "*.kap" | while read DIRFILE; do
      bsbfix "$DIRFILE"
   done
   # advise user
   echo
   echo " *** bsbfix completed, files dates are today, advice: make a rastow update ***"
   echo
   
   exit
}

#######################
# end bsbfix function #

#  bsbfix execution   #
#######################

[ "${METHOD}" = "bsb_fix" ] && bsb_fix ${IN_DIR}

########################
# end bsbfix execution #
########################



######################
# chartlist function #
######################

chart_list() {

   DIR="$1"
   # cleanup old tmp chartlist
   [ -f /tmp/chartlist.tmp ] && rm /tmp/chartlist.tmp

   # print header
   echo >"$DIR"/chartlist.txt
   echo "   #################################################" >>"$DIR"/chartlist.txt
   echo "   ### Chartlist generated by rastow.sh - GPL 3+ ###" >>"$DIR"/chartlist.txt
   echo "   #################################################" >>"$DIR"/chartlist.txt
   echo "             http://marinux.tuxfamily.org" >>"$DIR"/chartlist.txt
   echo >>"$DIR"/chartlist.txt
   echo " PATH/FILENAME         NW corner      SE corner      UPDATE      SCALE    DATUM    ZONE" >>"$DIR"/chartlist.txt
   echo >>"$DIR"/chartlist.txt


   # scan for kap files
   ####################
   find -L "$DIR" -iname "*.kap" | while read DIRFILE; do
      # limit lines header search
      # more of 640 in 1644DH01.kap (Brazil) founded
      #echo $DIRFILE
      head -n 680 ${DIRFILE} | iconv -f ISO_8859-1 |
      # extract datas
      gawk -F "," -v in_dir="$DIR" -v dirfile=${DIRFILE} \
      \
      '$1~/BSB\/NA=/ {
         # chart title
         #print $1
         # upper/lower case very irregular, so capitalize all tiles
         lg_title = length($1)
         title = toupper(substr($1, 8, (lg_title - 7)))
      }

      $2~/GD=/ {
         # chart datum
         #print $2
         datum = substr($2, 4)
      }

      $3~/ED=/ {
         # update
         if (nd != "found") update_yyyy($3)
      }

      $2~/ND=/ {
         # other update prefix (Brasilian charts)
         nd = found
         update_yyyy($2)
      }

      # scale
      $1~/KNP\/SC=/ {
         #print $1
         lg_scale = length($1)
         scale = "1/"substr($1, 8, (lg_scale - 7))
      }

      # corners NW and SE
      $1~/PLY\// {
         #print $1
         for (i = 2; i <= 4 ; i = i + 2) {
            if ($1 == "PLY/"i) {
               if ($2 > 0) lat[i] = mn_sex($2)"N"
               if ($2 < 0) lat[i] = mn_sex($2)"S"
               if ($2 == 0) lat[i] = mn_sex($2)"-"
               if ($3 > 0) lgt[i] = mn_sex($3)"E"
               if ($3 < 0) lgt[i] = mn_sex($3)"W"
               if ($3 == 0) lgt[i] = mn_sex($3)"-"
            }
         }
      }

      # conversion decimals degrees to sexagesimal degrees
      function mn_sex(deg_conv) {
         if (deg_conv < 0) deg_conv = deg_conv * -1
         deg = int(deg_conv)
         decim = int((deg_conv - deg) * 100)
         decim = int(decim * 0.6)
         if (length(decim) == 1) decim = "0"decim
         if (decim == 0) decim = "00"
         return deg"°"decim
      }

      # year format
      function update_yyyy(date) {

         # standard date format: yyyy-mm-dd
         if ((substr(date,8,1) == "-") && (substr(date,11,1) == "-")) {
            yyyy = substr(date, 4, 4)
            upd = "accurate"
         }

         # other format: dd/mm/yyyy or d/mm/yy or dd/m/yy etc...
         else {
            # look for day separator
            dd_sep = index(date,"/")
            # look for month separator
            mm_dd = substr(date,(dd_sep + 1))
            mm_sep = index(mm_dd,"/")
            # extract year
            yy = substr(mm_dd,(mm_sep + 1))
            # convert to numeric
            yy = yy + 0
            lg_yy = length(yy)
            # format year on 4 digits, year 2000
            if (lg_yy == 1) yyyy = 2000
            # only 2 digits given, arbitrary base on 1950
            else if (lg_yy == 2) {
               if (yy > 50) yyyy = "19" yy
               else yyyyyy = "20" yy
            }
            else if (lg_yy == 4) yyyy = yy
            #print yyyy
            upd = "accurate"
         }
         # constitution update
         if (upd == "accurate") update = yyyy
         else update = "unknow"
         #return
         #print update
         }


      END {
      # advise user
      #print title
      # format path file
      lg_indir = length(in_dir)
      lg_dirfile = length(dirfile)
      path_file = substr(dirfile, (lg_indir + 1) , lg_dirfile)
      # write chart datas
      print path_file" | "lat[2]" "lgt[2]" | "lat[4]" "lgt[4]" | "update" | "scale" | "datum" | "title >> "/tmp/chartlist.tmp"
      }'


   done


   # classify and write final list
   ###############################
   # don't harm
   LC_ALL=C

   sort -V /tmp/chartlist.tmp >>"$DIR"/chartlist.txt

   # advise user
   echo
   [ -f "$DIR"/chartlist.txt ] && echo "   ### chartlist.txt created in "$DIR" directory ###"
   echo

   exit
}

#############################
#  end chartlist function   #

# chartlist execution alone #
#############################

[ "${METHOD}" = "chart_list" ] && chart_list ${IN_DIR}

#################################
# end chartlist execution alone #
#################################



#########################
# copy / link execution #
#########################

# cleanup old classification
[ -f /tmp/class_1 ] && rm /tmp/class_1
[ -f /tmp/class_2 ] && rm /tmp/class_2

# create output directories
###########################
[ -d ${OUT_DIR} ] || mkdir -p ${OUT_DIR}

for i in N S ;do
   for j in 45+ 45- ;do
      for k in + - ;do
         for l in 01 02 03 04 05 06 07 08 09 10 11 ;do
            [ -d ${OUT_DIR}/${i}${j}TZ${k}${l} ] || mkdir ${OUT_DIR}/${i}${j}TZ${k}${l} >/dev/null
            if [ $? -ne 0 ]; then
               echo
               echo " *** unable to create output directories"
               echo " *** do you have the rights for this path ? ***"
               exit
            fi
         done
      done
      for m in 00 12 ;do
         [ -d ${OUT_DIR}/${i}${j}TZ=${m} ] || mkdir ${OUT_DIR}/${i}${j}TZ=${m}  >/dev/null
            if [ $? -ne 0 ]; then
               echo
               echo " *** unable to create output directories"
               echo " *** do you have the rights for this path ? ***"
               exit
            fi
      done
   done
done


# operations on  kap files
##########################
echo
echo " *** WORKING HARD *** "
echo
sleep 2
# listing
find ${IN_DIR} -iname "*.kap" | while read IN_DIRFILE; do
   IN_FILE=$(basename ${IN_DIRFILE})
   # make counter
   CNT_IN=$((CNT_IN += 1))
   # look for chart datas in header file, limit max found is 1644DH01.kap (Brazil - line 626)
   head -n 680 ${IN_DIRFILE} | iconv -f ISO_8859-1 |
   gawk -F "," -v in_dirfile=${IN_DIRFILE} -v in_file=${IN_FILE} -v out_dir=${OUT_DIR} -v method=${METHOD} -v cnt_in=${CNT_IN} \
      \
      '$1~/BSB\/NA=/ {
      # chart title
      #############
      lg_title = length($1)
      title = substr($1, 8, (lg_title - 7))
      # simple quote cause troubles replaced by "`"
      gsub("\x27", "\x60", title)
      }


      $1~/PLY\// {
         # operations on 4 corners
         #########################
         for (k = 1; k <= 4 ; k++) {
            if ($1 == "PLY/"k) {
               lat_[k] = $2
               lgt_[k] = $3
               # some USA charts ignore that there is an east world side...
               if (lgt_[k] < -180) lgt_[k] = 360 + lgt_[k]
               #print lat_[k] " " lgt_[k]
            }

            # select hemisphere
            if (lat_[k] >= 45) hemi_[k] = "N45+"
            if ((lat_[k] < 45) && (lat_[k] > 0)) hemi_[k] = "N45-"
            if ((lat_[k] > -45) && (lat_[k] < 0)) hemi_[k] = "S45-"
            if (lat_[k] <= -45) hemi_[k] = "S45+"

            # select timezone
            if (lgt_[k] > 0) zone_[k] = int((lgt_[k] + 7.5) /15)
            if (lgt_[k] < 0) zone_[k] = int((lgt_[k] - 7.5) /15)

            #format and build commands
            if (hemi_[k] != "") fmt_build(zone_[k], hemi_[k])

         }
      }


      END {
         # user informations
         ###################
         print "file number: "cnt_in
         print "title: "title
         # cases where PLY not found
         for (k = 1; k <= 4 ; k++) {
            if ((lat_[k] == "") || (lgt_[k] == "")) print "No PLY/"k" in "in_dirfile
         }

         # limits and exceptions
         #######################
         ## if lat from W to E are not equals, keep the greatest value
         # lat north
         if ((hemi_[4] > 0) && (hemi_[4] > hemi_[1])) hemi_[1] = hemi_[4]
         if ((hemi_[3] > 0) && (hemi_[3] > hemi_[2])) hemi_[2] = hemi_[3]
         # lat south
         if ((hemi_[4] < 0) && (hemi_[4] < hemi_[1])) hemi_[1] = hemi_[4]
         if ((hemi_[3] < 0) && (hemi_[3] < hemi_[2])) hemi_[2] = hemi_[3]

         # lat null, get the hemisphere from the opposite corner lat
         if (lat_[1] == 0) hemi_[1] = (substr(hemi_[2],1,1))"45-"
         if (lat_[2] == 0) hemi_[2] = (substr(hemi_[1],1,1))"45-"
         if (lat_[3] == 0) hemi_[3] = (substr(hemi_[4],1,1))"45-"
         if (lat_[4] == 0) hemi_[4] = (substr(hemi_[3],1,1))"45-"

         # intermediate zones function call for charts larger than one TZ
         ################################################################
         tz_larg(zone_[1], zone_[4], hemi_[1])
         tz_larg(zone_[2], zone_[3], hemi_[2])

         # hemispheres function call for charts larger than 60° in lat
         #############################################################
         # first possibility
         if ((hemi_[1] == "S""45""+") && (hemi_[2] == "N""45""+")) {
            tz_larg(zone_[1], zone_[4], "S45-")
            tz_larg(zone_[2], zone_[3], "S45-")
            tz_larg(zone_[1], zone_[4], "N45-")
            tz_larg(zone_[2], zone_[3], "N45-")
         }
         # second possibility
         if ((hemi_[1] == "S""45""+") && (hemi_[2] == "N""45""-")) {
            tz_larg(zone_[1], zone_[4], "S45-")
            tz_larg(zone_[2], zone_[3], "S45-")
         }
         # third possibility
         if ((hemi_[1] == "S""45""-") && (hemi_[2] == "N""45""+")) {
            tz_larg(zone_[1], zone_[4], "N45-")
            tz_larg(zone_[2], zone_[3], "N45-")
         }
      }

      # intermediates zones calculations in tz space
      ##############################################
      function tz_larg(zone_a, zone_b, hemi_c) {
         # case on meridian date change (tz=+12, tz=-12)
         if ((zone_a > 0) && (zone_b < 0)) cov_ = (12 - zone_a) + (12 + zone_b)
         # all others cases
         else cov_ = (zone_b - zone_a)
         if (cov_ > 1) {
            # create intermediate timezones
            for (i = 0; i <= cov_; i++) {
               zone_a = (zone_b - i)
               # adjust on base 12
               if (zone_a > 12) zone_a = -12 + (zone_a - 12)
               if (zone_a < -12) zone_a = 12 + (zone_a + 12)
               #format and build commands
               fmt_build(zone_a, hemi_c)
            }
         }
       }


      # format and build commands
      ###########################
      function fmt_build(zone_, hemi_) {

         # mix TZ+12 and TZ-12, shared on the 180° meridian
         if ((zone_ == 12 ) || (zone_ == -12 )) zone_ = "=""12"
         # mix TZ+O and TZ-0, shared on the 0° meridian
         if (zone_ == 0 ) zone_ = "=""00"

         # format timezone for subdirs created
         if (zone_ > 0) {
            if (length(zone_) == 1) zone_ = "+0"zone_
            else if (length(zone_) == 2) zone_ = "+"zone_
         }
         if ((zone_ < 0) && (length(zone_) == 2)) zone_ = "-0"(zone_ * -1)

         # build instructions to copy/link
         if (method == "copy") cmd_ = "cp -u -p "in_dirfile" "out_dir"/"hemi_"TZ"zone_"/"in_file
         if (method == "sym_link") cmd_ = "ln -s -f "in_dirfile" "out_dir"/"hemi_"TZ"zone_"/"in_file

         # for user information show zone with their correpondant charts
         inf_ = " && echo " "\x27" hemi_"TZ"zone_ " \x3C\x3C\x3C " title "\x27"
         # constitution cmdline
         print cmd_ inf_ >> "/tmp/class_1"

       }'

done


# create final command file
###########################
# don't harm
LC_ALL=C
# remove adjacent duplicate lines and the mysterious CR apparition
sort /tmp/class_1 | uniq | tr -d '\r' >/tmp/class_2
# count number entries
nb_lgn=$(cat /tmp/class_2 | wc -l)


# execute command file
######################
# advise user
echo
[ -e /tmp/class_2 ] && echo " *** COPY / LINK *.KAP FILES *** "
echo
sleep 5
# copy/link *.kap in their TZ
sh /tmp/class_2
# advise user
echo
[ -e /tmp/class_2 ] && echo " *** $nb_lgn ENTRIES OK *** "
echo


# create chartlist.txt
######################
chart_list ${OUT_DIR}


#############################
# end copy / link execution #
#############################

